"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProjectPackageImports = exports.watchCompileAsync = exports.compileAsync = exports.performCompilationAsync = void 0;
const ng = __importStar(require("@angular/compiler-cli"));
const typescript_1 = __importDefault(require("typescript"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const typeCheck_1 = require("./typeCheck");
/**
 * Based on `@angular/compiler-cli.performCompilation()`
 */
exports.performCompilationAsync = async ({ compilerHost, compilerOptions, rootNames, oldProgram }, typeCheck = true) => {
    const diagnostics = [];
    const program = ng.createProgram({
        rootNames,
        host: compilerHost,
        options: compilerOptions,
        oldProgram,
    });
    await program.loadNgStructureAsync();
    if (typeCheck)
        diagnostics.push(...typeCheck_1.runTypeCheck(rootNames, compilerOptions, program, compilerHost));
    const emitResult = program.emit();
    diagnostics.push(...emitResult.diagnostics);
    return { diagnostics, program, emitResult };
};
exports.compileAsync = async ({ rootNames, compilerHost, compilerOptions, }) => {
    const compilationResult = await exports.performCompilationAsync({
        rootNames,
        compilerHost,
        compilerOptions,
    });
    return compilationResult;
};
exports.watchCompileAsync = async ({ rootNames, compilerHost, compilerOptions, }) => {
    const compiledFiles = new Set();
    const fileCache = new Map();
    const modifiedFile = new Set();
    let cachedProgram;
    const getCacheEntry = (fileName) => {
        fileName = path_1.default.normalize(fileName);
        let entry = fileCache.get(fileName);
        if (!entry) {
            entry = {};
            fileCache.set(fileName, entry);
        }
        return entry;
    };
    // Setup compilerHost to use cache
    const oriWriteFile = compilerHost.writeFile;
    compilerHost.writeFile = (fileName, data, writeByteOrderMark, onError, sourceFiles) => {
        const resolvedPath = path_1.default.resolve(fileName);
        compiledFiles.add(resolvedPath);
        return oriWriteFile(fileName, data, writeByteOrderMark, onError, sourceFiles);
    };
    const oriFileExists = compilerHost.fileExists;
    compilerHost.fileExists = (fileName) => {
        const cache = getCacheEntry(fileName);
        if (cache.exists === null || cache.exists === undefined)
            cache.exists = oriFileExists(fileName);
        return cache.exists;
    };
    const oriGetSourceFile = compilerHost.getSourceFile;
    compilerHost.getSourceFile = (fileName, languageVersion) => {
        const cache = getCacheEntry(fileName);
        if (!cache.sf)
            cache.sf = oriGetSourceFile(fileName, languageVersion);
        return cache.sf;
    };
    const oriReadFile = compilerHost.readFile;
    compilerHost.readFile = (fileName) => {
        const cache = getCacheEntry(fileName);
        if (!cache.content)
            cache.content = oriReadFile(fileName);
        return cache.content;
    };
    // Read resource is a optional function,
    // it has priority over readFile when loading resources (html/css),
    // async file processing will require a custom performCompilation to run `program.loadNgStuctureAsync()`
    const oriReadResource = compilerHost.readResource;
    if (oriReadResource)
        compilerHost.readResource = async (fileName) => {
            const cache = getCacheEntry(fileName);
            if (cache.content === undefined)
                cache.content = await oriReadResource(fileName);
            return cache.content;
        };
    compilerHost.getModifiedResourceFiles = () => {
        return modifiedFile;
    };
    // Do first compile
    const firstCompilation = await exports.compileAsync({
        rootNames,
        compilerHost,
        compilerOptions,
    });
    cachedProgram = firstCompilation.program;
    const recompile = async (fileName) => {
        // perhaps this function need debouncing like in perform_watch.ts
        fileName = path_1.default.normalize(fileName);
        fileCache.delete(fileName);
        modifiedFile.add(fileName);
        compiledFiles.clear();
        const oldProgram = cachedProgram;
        cachedProgram = undefined;
        const recompileResult = await exports.performCompilationAsync({
            rootNames,
            compilerHost,
            compilerOptions,
            oldProgram,
        }, false);
        cachedProgram = recompileResult.program;
        modifiedFile.clear();
        return {
            program: recompileResult.program,
            emitResult: recompileResult.emitResult,
            recompiledFiles: [...compiledFiles],
            diagnostics: recompileResult.diagnostics,
        };
    };
    return { firstCompilation, recompile };
};
exports.getProjectPackageImports = (rootNames, options) => {
    const host = ng.createCompilerHost({ options });
    const program = ng.createProgram({
        rootNames,
        host,
        options,
    });
    const packages = new Set();
    for (const source of program.getTsProgram().getSourceFiles()) {
        if (!source.fileName.includes('node_modules')) {
            source.forEachChild((child) => {
                if (typescript_1.default.isImportDeclaration(child)) {
                    try {
                        if (typescript_1.default.isStringLiteral(child.moduleSpecifier)) {
                            const packageName = child.moduleSpecifier.text;
                            fs_1.default.statSync(path_1.default.join('node_modules', packageName));
                            packages.add(packageName);
                        }
                    }
                    catch (err) { }
                }
            });
        }
    }
    return packages;
};
