"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AngularCompilerService = void 0;
const configParser_1 = require("./configParser");
const ng = __importStar(require("@angular/compiler-cli"));
const ngcc = __importStar(require("@angular/compiler-cli/ngcc"));
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const styleResource_1 = require("./styleResource");
const compile_1 = require("./compile");
const typeCheckWorker_1 = require("./typeCheckWorker");
const typeCheck_1 = require("./typeCheck");
class AngularCompilerService {
    constructor(angularJson, ngccTargets, snowpackConfig, angularProject) {
        this.ngccTargets = ngccTargets;
        this.snowpackConfig = snowpackConfig;
        this.angularProject = angularProject;
        this._builtFiles = new Map();
        this._buildStatus = {
            built: false,
            building: false,
            buildReadyCallback: [],
        };
        this._typeCheckErrorListeners = new Map();
        this._typeCheckErrorListenerId = 0;
        this._lastTypeCheckResult = [];
        this._fileReplacements = new Map();
        this._cwd = path_1.default.resolve(process.cwd());
        this._isWatch = false;
        this._angularConfig = configParser_1.readAngularJson(angularJson);
        this._tsConfig = this._angularConfig.getResolvedFilePaths(angularProject).tsConfig;
        this._ngConfiguration = ng.readConfiguration(this._tsConfig);
        this._ngCompilerHost = this.configureCompilerHost(ng.createCompilerHost({ options: this._ngCompilerOptions }));
        this._ngCompilerOptions.outDir = undefined;
    }
    get _ngCompilerOptions() {
        return this._ngConfiguration.options;
    }
    async ngcc(target) {
        console.debug(`[angular] ngcc processing: ${target}`);
        await ngcc.process({
            async: true,
            basePath: this._cwd,
            tsConfigPath: path_1.default.resolve(this._tsConfig),
            targetEntryPointPath: path_1.default.resolve(path_1.default.join('node_modules', target)),
        });
    }
    configureCompilerHost(host) {
        host.writeFile = (fileName, contents) => {
            fileName = path_1.default.resolve(fileName);
            // If sourceMap is inlined, leave it in the source.
            if (!this._ngCompilerOptions.inlineSourceMap)
                contents = contents.replace(/\/\/# sourceMappingURL.*/, '');
            this._builtFiles.set(fileName, contents);
        };
        host.readResource = async (fileName) => {
            // If not reading a style, ie: template, just read from disk
            // or when the snowpack dev server was not started through `ngsnow` (contains code to make preprocess work)
            if (!fileName.match(styleResource_1.STYLES_FILEEXT_REGEX) ||
                (this._isWatch && !styleResource_1.styleResourceManager.hasListener))
                return await fs_1.promises.readFile(fileName, 'utf-8');
            else
                return await styleResource_1.styleResourceManager.requestStyle(fileName);
        };
        const oriReadFile = host.readFile;
        if (process.env.NODE_ENV === 'production') {
            const replacementConfig = this.angularConfig.getProject(this.angularProject).architect.build.configurations.production.fileReplacements;
            replacementConfig.forEach((replacement) => {
                this._fileReplacements.set(path_1.default.resolve(replacement.replace), path_1.default.resolve(replacement.with));
            });
        }
        host.readFile = (fileName) => {
            fileName = path_1.default.resolve(fileName);
            if (this._fileReplacements.has(fileName)) {
                const replaceWith = this._fileReplacements.get(fileName);
                fileName = replaceWith;
            }
            return oriReadFile(fileName);
        };
        return host;
    }
    registerTypeCheckWorker() {
        this._typeCheckWorker = typeCheckWorker_1.createTypeCheckWorker();
        this._typeCheckWorker.on('message', (msg) => {
            this._lastTypeCheckResult = msg;
            this._typeCheckErrorListeners.forEach((listener) => listener(msg));
        });
    }
    async buildSource(watch) {
        if (this._buildStatus.built)
            return;
        else if (!this._buildStatus.built && !this._buildStatus.building) {
            this._buildStatus.building = true;
            this._isWatch = watch;
            if (watch && !styleResource_1.styleResourceManager.hasListener) {
                console.warn('[angular] styleResourceManager has no listener, run "ngsnow" to enable style preprocessing.');
            }
            const compileArgs = {
                rootNames: this._ngConfiguration.rootNames,
                compilerHost: this._ngCompilerHost,
                compilerOptions: this._ngCompilerOptions,
            };
            const importedPackages = compile_1.getProjectPackageImports(this._ngConfiguration.rootNames, this._ngCompilerOptions);
            importedPackages.add('@angular/common');
            this.ngccTargets.forEach((t) => importedPackages.add(t));
            for (const target of importedPackages) {
                await this.ngcc(target);
            }
            if (watch) {
                console.warn('[angular] Try clearing snowpack development cache with "snowpack --reload" if facing errors during dev mode');
                this.registerTypeCheckWorker();
                const result = await compile_1.watchCompileAsync(compileArgs);
                this._recompileFunction = result.recompile;
                this._lastCompilationResult = result.firstCompilation;
            }
            else {
                this._lastCompilationResult = await compile_1.compileAsync(compileArgs);
            }
            this._buildStatus.built = true;
            this._buildStatus.building = false;
            this._buildStatus.buildReadyCallback.forEach((cb) => cb());
            this._buildStatus.buildReadyCallback = [];
        }
        else {
            return new Promise((resolve) => {
                this._buildStatus.buildReadyCallback.push(resolve);
            });
        }
    }
    async recompile(modifiedFile) {
        if (!this._recompileFunction)
            throw new Error('Cannot recompile as angular was not build with watch mode enabled');
        else {
            const recompiledResult = await this._recompileFunction(modifiedFile);
            this._lastCompilationResult = {
                diagnostics: recompiledResult.diagnostics,
                emitResult: recompiledResult.emitResult,
                program: recompiledResult.program,
            };
            this._lastTypeCheckResult = [];
            const workerMessage = {
                action: 'run_check',
                data: {
                    options: this._ngCompilerOptions,
                    rootNames: this._ngConfiguration.rootNames,
                },
            };
            this._typeCheckWorker.postMessage(workerMessage);
            const recompiledFiles = recompiledResult.recompiledFiles
                .filter((file) => path_1.default.extname(file) === '.js')
                .map((file) => path_1.default.resolve(file).replace(path_1.default.extname(file), '.ts'));
            return {
                recompiledFiles,
                recompiledResult,
            };
        }
    }
    getBuiltFile(filePath) {
        filePath = path_1.default.resolve(filePath);
        let result = null;
        const codeFile = filePath.replace(path_1.default.extname(filePath), '.js');
        const mapFile = filePath.replace(path_1.default.extname(filePath), '.js.map');
        if (this._builtFiles.has(codeFile)) {
            result = {
                code: this._builtFiles.get(codeFile),
            };
            if (this._builtFiles.has(mapFile))
                result.map = this._builtFiles.get(mapFile);
        }
        return result;
    }
    onTypeCheckError(listener) {
        const id = this._typeCheckErrorListenerId++;
        this._typeCheckErrorListeners.set(id, listener);
        return id;
    }
    removeTypeCheckErrorListener(id) {
        this._typeCheckErrorListeners.delete(id);
    }
    getIndexInjects() {
        const { index, styles, scripts, main, polyfills, } = this._angularConfig.getResolvedFilePaths(this.angularProject);
        const indexDir = path_1.default.dirname(index);
        const injectStyles = styles.map((style) => {
            const relativeFromRoot = path_1.default.relative(this._cwd, style);
            if (relativeFromRoot.startsWith('node_modules')) {
                const webModuleStylePath = path_1.default
                    .join(this.snowpackConfig.buildOptions.metaUrlPath, relativeFromRoot.replace('node_modules', 'pkg'))
                    .replace(path_1.default.extname(style), '.css');
                return `<link rel="stylesheet" href="${webModuleStylePath}">`;
            }
            else {
                const relativeUrl = path_1.default
                    .relative(indexDir, style)
                    .replace(path_1.default.extname(style), '.css');
                return `<link rel="stylesheet" href="${relativeUrl}">`;
            }
        });
        const injectScripts = scripts.map((script) => {
            const relativeFromRoot = path_1.default.relative(this._cwd, script);
            if (relativeFromRoot.startsWith('node_modules')) {
                const webModuleScriptPath = path_1.default
                    .join(this.snowpackConfig.buildOptions.metaUrlPath, relativeFromRoot.replace('node_modules', 'pkg'))
                    .replace(path_1.default.extname(script), '.js');
                return `<script defer type="module" src="${webModuleScriptPath}"></script>`;
            }
            else {
                const relativeUrl = path_1.default
                    .relative(indexDir, script)
                    .replace(path_1.default.extname(script), '.js');
                return `<script defer type="module" src="${relativeUrl}"></script>`;
            }
        });
        const relativePolyfillsUrl = path_1.default
            .relative(indexDir, polyfills)
            .replace(path_1.default.extname(polyfills), '.js');
        const injectPolyfills = `<script defer type="module" src="${relativePolyfillsUrl}"></script>`;
        const relativeMainUrl = path_1.default
            .relative(indexDir, main)
            .replace(path_1.default.extname(main), '.js');
        const injectMain = `<script defer type="module" src="${relativeMainUrl}"></script>`;
        return {
            injectPolyfills,
            injectMain,
            injectStyles,
            injectScripts,
        };
    }
    getErrorInFile(filePath) {
        return typeCheck_1.getTSDiagnosticErrorInFile(filePath, [
            ...this._lastTypeCheckResult,
            ...this._lastCompilationResult.diagnostics,
        ]);
    }
    getErroredFiles(diagnostics) {
        return typeCheck_1.getTSDiagnosticErrorFile(diagnostics);
    }
    formatDiagnostics(diagnostics) {
        return ng.formatDiagnostics(diagnostics, typeCheck_1.tsFormatDiagnosticHost);
    }
    get ngConfiguration() {
        return this._ngConfiguration;
    }
    get angularConfig() {
        return this._angularConfig;
    }
    getAngularCriticalFiles() {
        return this._angularConfig.getResolvedFilePaths(this.angularProject);
    }
    useSourceMaps(state, inlineSources) {
        if (inlineSources !== undefined)
            this._ngCompilerOptions.inlineSources = inlineSources;
        switch (state) {
            case 'none':
                this._ngCompilerOptions.sourceMap = this._ngCompilerOptions.inlineSourceMap = false;
                this._ngCompilerOptions.inlineSources = false;
                break;
            case 'normal':
                this._ngCompilerOptions.sourceMap = true;
                this._ngCompilerOptions.inlineSourceMap = false;
                break;
            case 'inline':
                this._ngCompilerOptions.inlineSourceMap = true;
                this._ngCompilerOptions.sourceMap = false;
                break;
        }
    }
}
exports.AngularCompilerService = AngularCompilerService;
