"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.styleResourceManager = void 0;
const compilerService_1 = require("./compilerService");
const path_1 = __importDefault(require("path"));
const typescript_1 = require("typescript");
const styleResource_1 = require("./styleResource");
var styleResource_2 = require("./styleResource");
Object.defineProperty(exports, "styleResourceManager", { enumerable: true, get: function () { return styleResource_2.styleResourceManager; } });
const pluginFactory = (snowpackConfig, pluginOptions) => {
    var _a, _b;
    const angularJson = (pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.angularJson) || 'angular.json';
    const angularProject = pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.angularProject;
    const errorToBrowser = (_a = pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.errorToBrowser) !== null && _a !== void 0 ? _a : true;
    const useHmr = (_b = pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.useHmr) !== null && _b !== void 0 ? _b : false;
    const ngccTargets = (pluginOptions === null || pluginOptions === void 0 ? void 0 : pluginOptions.ngccTargets) || [];
    const useSourceMaps = snowpackConfig.buildOptions.sourcemap; // For snowpack 3
    ngccTargets.unshift('@angular/core', '@angular/common', '@angular/platform-browser-dynamic');
    const compiler = new compilerService_1.AngularCompilerService(angularJson, ngccTargets, snowpackConfig, angularProject);
    const skipRecompileFiles = [];
    const { index, main, scripts, styles } = compiler.getAngularCriticalFiles();
    compiler.useSourceMaps(useSourceMaps ? 'normal' : 'none', true);
    const knownEntrypoints = ['@angular/common'];
    if (useHmr) {
        knownEntrypoints.push('angular-snowpack-plugin/vendor/hmr/hmr-accept');
        if (useSourceMaps)
            compiler.useSourceMaps('inline', true);
    }
    // Styles and scripts from npm packages
    [...scripts, ...styles]
        .map((filePath) => path_1.default.relative(process.cwd(), filePath))
        .filter((filePath) => filePath.startsWith('node_modules'))
        .forEach((filePath) => knownEntrypoints.push(path_1.default.relative(path_1.default.resolve('node_modules'), path_1.default.resolve(filePath))));
    const plugin = {
        name: 'angular-snowpack-plugin',
        knownEntrypoints,
        resolve: {
            input: ['.ts'],
            output: ['.js'],
        },
        config() {
            compiler.onTypeCheckError((diagnostic) => {
                if (diagnostic.length > 0) {
                    if (!errorToBrowser) {
                        console.error(`[angular] ${compiler.formatDiagnostics(diagnostic)}`);
                    }
                    else {
                        const erroredFiles = compiler.getErroredFiles(diagnostic);
                        skipRecompileFiles.push(...erroredFiles);
                        erroredFiles.forEach((file) => this.markChanged(file));
                    }
                }
            });
        },
        async load({ filePath, isDev, isHmrEnabled }) {
            if (useHmr &&
                isHmrEnabled &&
                compiler.ngConfiguration.options.module !== typescript_1.ModuleKind.ESNext)
                compiler.ngConfiguration.options.module = typescript_1.ModuleKind.ESNext;
            await compiler.buildSource(isDev || isHmrEnabled);
            const error = compiler.getErrorInFile(filePath);
            if (error.length > 0)
                throw new Error(`[angular] ${compiler.formatDiagnostics(error)}`);
            let result = compiler.getBuiltFile(filePath);
            if (useHmr && isHmrEnabled && result && path_1.default.resolve(filePath) === main) {
                result = Object.assign({}, result);
                result.code = `import hmrAccept from 'angular-snowpack-plugin/vendor/hmr/hmr-accept';\n${result.code}\nif(import.meta.hot) hmrAccept(import.meta);\n`;
            }
            return {
                '.js': {
                    code: result === null || result === void 0 ? void 0 : result.code,
                    map: useSourceMaps ? result === null || result === void 0 ? void 0 : result.map : undefined,
                },
            };
        },
        async onChange({ filePath }) {
            filePath = path_1.default.resolve(filePath);
            if (skipRecompileFiles.includes(filePath)) {
                skipRecompileFiles.splice(skipRecompileFiles.indexOf(filePath), 1);
                return;
            }
            styleResource_1.styleResourceManager.purgeCache(filePath);
            const recompile = await compiler.recompile(filePath);
            recompile.recompiledFiles = recompile.recompiledFiles.filter((file) => file !== filePath);
            skipRecompileFiles.push(...recompile.recompiledFiles);
            recompile.recompiledFiles.forEach((file) => this.markChanged(file));
        },
        async transform({ id, contents, fileExt }) {
            id = path_1.default.resolve(id);
            if (id === index) {
                const { injectMain, injectPolyfills, injectScripts, injectStyles, } = compiler.getIndexInjects();
                return contents
                    .toString('utf-8')
                    .replace('</head>', `${injectStyles.join('')}\n</head>`)
                    .replace('</body>', `${injectPolyfills}${injectMain}${injectScripts.join('')}\n</body>`);
            }
            else if (fileExt.match(styleResource_1.STYLES_FILEEXT_REGEX)) {
                styleResource_1.styleResourceManager.submitStyle(id, contents.toString('utf-8'));
            }
        },
    };
    return plugin;
};
exports.default = pluginFactory;
