/// <reference types="node" />
import { EventEmitter } from 'events';
interface StyleResourceEvents {
    request: (stylePath: string) => void;
}
declare interface StyleResource {
    on<U extends keyof StyleResourceEvents>(event: U, listener: StyleResourceEvents[U]): this;
    emit<U extends keyof StyleResourceEvents>(event: U, ...args: Parameters<StyleResourceEvents[U]>): boolean;
}
declare class StyleResource extends EventEmitter {
    private styleLookups;
    constructor();
    /**
     * Returns a promise of the compiled style
     * @param stylePath Path to the style requested
     *
     * The compiled style will be provided from the `transform` step of the plugin.
     */
    requestStyle(stylePath: string): Promise<string>;
    submitStyle(stylePath: string, styleContents: string): void;
    purgeCache(stylePath?: string): void;
    get hasListener(): boolean;
}
export declare const styleResourceManager: StyleResource;
export declare const STYLES_FILEEXT_REGEX: RegExp;
export {};
