"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.STYLES_FILEEXT_REGEX = exports.styleResourceManager = void 0;
const events_1 = require("events");
const path_1 = __importDefault(require("path"));
class StyleResource extends events_1.EventEmitter {
    constructor() {
        super();
        this.styleLookups = new Map();
    }
    /**
     * Returns a promise of the compiled style
     * @param stylePath Path to the style requested
     *
     * The compiled style will be provided from the `transform` step of the plugin.
     */
    async requestStyle(stylePath) {
        stylePath = path_1.default.resolve(stylePath);
        this.emit('request', stylePath);
        return new Promise((resolve) => {
            const compiledStylePath = stylePath.replace(path_1.default.extname(stylePath), '.css');
            if (!this.styleLookups.has(compiledStylePath)) {
                this.styleLookups.set(compiledStylePath, {
                    listeners: [],
                    content: null,
                });
            }
            const lookup = this.styleLookups.get(compiledStylePath);
            if (typeof lookup.content === 'string') {
                resolve(lookup.content);
            }
            else {
                lookup.listeners.push(resolve);
            }
        });
    }
    submitStyle(stylePath, styleContents) {
        stylePath = path_1.default
            .resolve(stylePath)
            .replace(path_1.default.extname(stylePath), '.css');
        if (!this.styleLookups.has(stylePath)) {
            this.styleLookups.set(stylePath, {
                listeners: [],
                content: null,
            });
        }
        const lookup = this.styleLookups.get(stylePath);
        lookup.content = styleContents;
        for (const listener of lookup.listeners) {
            listener(styleContents);
        }
        lookup.listeners = [];
    }
    purgeCache(stylePath) {
        if (!stylePath) {
            this.styleLookups.clear();
        }
        else {
            stylePath = path_1.default
                .resolve(stylePath)
                .replace(path_1.default.extname(stylePath), '.css');
            const lookup = this.styleLookups.get(stylePath);
            if (lookup)
                lookup.content = null;
        }
    }
    get hasListener() {
        return this.listenerCount('request') > 0;
    }
}
// Export singleton manager
exports.styleResourceManager = new StyleResource();
exports.STYLES_FILEEXT_REGEX = /\.(css|scss|sass|styl|less)$/;
